// Copyright 2020, 2021, 2022, 2023 Francesco Biscani (bluescarni@gmail.com), Dario Izzo (dario.izzo@gmail.com)
//
// This file is part of the heyoka library.
//
// This Source Code Form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

#ifndef HEYOKA_DETAIL_NUM_IDENTITY_HPP
#define HEYOKA_DETAIL_NUM_IDENTITY_HPP

#include <cstdint>
#include <vector>

#include <heyoka/config.hpp>
#include <heyoka/detail/fwd_decl.hpp>
#include <heyoka/detail/llvm_fwd.hpp>
#include <heyoka/detail/visibility.hpp>
#include <heyoka/func.hpp>
#include <heyoka/s11n.hpp>

HEYOKA_BEGIN_NAMESPACE

namespace detail
{

// NOTE: this function is used exclusively in a Taylor decomposition to replace
// elementary number subexpression that might be generated by custom taylor_decompose()
// implementations with a function that returns the number itself. This allows us to
// avoid special-casing such corner cases.
// NOTE: this could probably be easily extended to deal with elementary param subexpressions
// if ever needed.
class HEYOKA_DLL_PUBLIC num_identity_impl : public func_base
{
    friend class boost::serialization::access;
    template <typename Archive>
    void serialize(Archive &ar, unsigned)
    {
        ar &boost::serialization::base_object<func_base>(*this);
    }

public:
    num_identity_impl();
    explicit num_identity_impl(expression);

    llvm::Value *taylor_diff(llvm_state &, llvm::Type *, const std::vector<std::uint32_t> &,
                             const std::vector<llvm::Value *> &, llvm::Value *, llvm::Value *, std::uint32_t,
                             std::uint32_t, std::uint32_t, std::uint32_t, bool) const;

    llvm::Function *taylor_c_diff_func(llvm_state &, llvm::Type *, std::uint32_t, std::uint32_t, bool) const;
};

// NOTE: public only for testing.
HEYOKA_DLL_PUBLIC expression num_identity(expression);

} // namespace detail

HEYOKA_END_NAMESPACE

HEYOKA_S11N_FUNC_EXPORT_KEY(heyoka::detail::num_identity_impl)

#endif
